# -*- coding: utf-8 -*-
import json
import os
import argparse
from hashlib import md5
from plutonium.utils import logger, VoyagerDetect, zip_folder
from plutonium.config import (
    VOYAGER_SERVER,
    VOYAGER_USERNAME,
    VOYAGER_PASSWORD,
    VOYAGER_TOKEN,
    LOG_FILENAME,
    LOGO,
    DATA_DIR,
    AGENT_UUID
)
def init_parse():
    parser = argparse.ArgumentParser(
        description="SCA Agent based on Cdxgen and internal project Voyager I, for application dependencies and risk discovery。"
    )
    parser.add_argument(
        "-t",
        "--type",
        dest="type",
        required=True,
        help="project_sca/image_sca/package_sca/api/other/project_vul/image_vul/image_exec/image_detect",
    )
    # project_sca
    # image_sca
    # image_detect
    parser.add_argument(
        "-trigger-source",
        "--trigger-source",
        dest="trigger_source",
        help="Trigger Source",
    )
    parser.add_argument(
        "--scene-token",
        dest="scene_token",
        help="Scene token",
    )
    parser.add_argument(
        "-s",
        "--source",
        dest="source",
        required=True,
        help="Source directory or container image or binary file",
    )
    parser.add_argument(
        "--tool",
        dest="tool",
        help="Analysis Tool",
    )
    parser.add_argument(
        "--tool-opitons",
        dest="tool_options",
        help="Analysis Tool options",
    )
    parser.add_argument(
        "--firewall-status",
        dest="firewall_status",
        action="store_true",
        default=False,
        help="是否进行拦截检测",
    )
    parser.add_argument(
        "--is-sync",
        dest="is_sync",
        action="store_true",
        default=False,
        help="是否同步检测",
    )
    # 服务端参数
    parser.add_argument(
        "--voyager-server",
        default=VOYAGER_SERVER,
        dest="voyager_server",
        help="Voyager server url. Eg: https://api.voyager.com",
    )
    parser.add_argument(
        "--voyager-username",
        default=VOYAGER_USERNAME,
        dest="voyager_username",
        help="Voyager username",
    )
    parser.add_argument(
        "--voyager-password",
        default=VOYAGER_PASSWORD,
        dest="voyager_password",
        help="Voyager password",
    )
    parser.add_argument(
        "--voyager-token",
        default=VOYAGER_TOKEN,
        dest="voyager_token",
        help="Voyager token for token based submission",
    )
    parser.add_argument(
        "--voyager-api",
        default="",
        dest="voyager_api",
        help="Voyager api url",
    )
    parser.add_argument(
        "--analyze-uuid",
        dest="analyze_uuid",
        help="分析任务id",
    )
    # 项目参数信息
    parser.add_argument(
        "--project-name",
        dest="project_name",
        help="project name",
    )
    parser.add_argument(
        "--project-url",
        dest="project_url",
        help="project repository url",
    )
    parser.add_argument(
        "--project-user",
        dest="project_user",
        help="project user",
    )
    parser.add_argument(
        "--user-email",
        dest="user_email",
        help="操作用户邮箱",
    )
    parser.add_argument(
        "--project-branch",
        dest="project_branch",
        help="project branch",
    )
    parser.add_argument(
        "--project-file",
        dest="project_file",
        help="project file",
    )
    parser.add_argument(
        "--project-commit-id",
        dest="project_commit_id",
        help="project commit id",
    )
    #
    parser.add_argument(
        "--image-data-type",
        dest="image_data_type",
        help="镜像数据类型，image_name/image_file/image_dockerfile/image_analysis_file",
    )
    parser.add_argument(
        "--image-analysis-file",
        dest="image_analysis_file",
        help="镜像分析结果文件",
    )
    parser.add_argument(
        "--image-analysis-file-type-list",
        dest="image_analysis_file_type_list",
        help="镜像分析结果文件类型列表dockerfile--custom,sca--trivy,sca--syft,exec--custom,vul--trivy,vul--grype",
    )
    # 镜像参数信息
    parser.add_argument(
        "--image-name",
        dest="image_name",
        help="镜像名称",
    )
    parser.add_argument(
        "--image-full-name",
        dest="image_full_name",
        help="镜像完整名称",
    )
    parser.add_argument(
        "--image-id",
        dest="image_id",
        help="镜像Id",
    )
    parser.add_argument(
        "--image-type",
        dest="image_type",
        help="镜像类型,name/file/dockerfile",
    )
    parser.add_argument(
        "--image-file",
        dest="image_file",
        help="镜像文件",
    )
    parser.add_argument(
        "--image-sca-file",
        dest="image_sca_file",
        help="镜像SCA结果文件",
    )
    parser.add_argument(
        "--dockerfile-path",
        dest="dockerfile_path",
        help="dockerfile路径",
    )
    parser.add_argument(
        "--scan-in-server",
        dest="scan_in_server",
        action="store_true",
        default=False,
        help="是否在服务端扫描",
    )
    parser.add_argument(
        "--image-analysis-type",
        dest="image_analysis_type",
        help="sca/exec/vul/sca_vul/backdoor/other",
    )
    parser.add_argument(
        "--image-repository-url",
        dest="image_repository_url",
        help="镜像仓库地址",
    )
    parser.add_argument(
        "--extra-data",
        dest="extra_data",
        help="附加参数a=b&c=d",
    )
    parser.add_argument(
        "--error-try-count",
        dest="error_try_count",
        default=5,
        help="错误尝试次数",
    )
    parser.add_argument(
        "--cmd-time-out",
        dest="cmd_time_out",
        default=30,
        help="命令执行超时时间",
    )
    parser.add_argument(
        "--time-wait-for-report",
        dest="time_wait_for_report",
        default=20,
        help="等待检测报告时间",
    )
    # parser.print_help()
    return parser.parse_args()


def main():
    # print(LOGO)
    args = init_parse()
    data = {
        'type': args.type,
        'agent_uuid': AGENT_UUID,
        'trigger_source': args.trigger_source,
        'scene': args.scene_token,
        'tool': args.tool,
        'tool_options': args.tool_options,
        'firewall_status': args.firewall_status,
        'is_sync': args.is_sync,
        'analyze_uuid': args.analyze_uuid if args.analyze_uuid else '',
        # 项目信息
        # 镜像类
        'image_name': args.image_name,
        'image_full_name': args.image_full_name,
        'image_repository_url': args.image_repository_url,
        'image_analysis_type': args.image_analysis_type,
        'image_data_type': args.image_data_type,
        'image_type': args.image_type,
        'image_analysis_file': args.image_analysis_file,
        'image_analysis_file_type_list': args.image_analysis_file_type_list,
        # 项目类
        'project_name': args.project_name,
        'project_branch': args.project_branch,
        'project_user': args.project_user,
        'user_email': args.user_email,
        'project_commit_id': args.project_commit_id,
        'project_url': args.project_url,
        # 附加参数
        'extra_data': args.extra_data,
    }
    if not data['image_name']:
        if data.get('image_full_name'):
            data['image_name'] = data.get('image_full_name', '').split(':')[0]
    attach_files = [
        # 提交的名称前缀需要与scan_log_detail的type一致
        # 项目文件
        # ('log_file', (SCA, open('./todo.md', 'rb'),)),
        # ('core_files_list', ('pom.xml', open('./todo.md', 'rb'), )),
        # ('core_files_list', ('package.json.lock', open('./todo.md', 'rb'), )),
        # ('sbom_files_list', ('sca_cdxgen.json', open('./voyager.json', 'rb'),)),
        # ('sbom_files_list', ('sca_dependency_tree.txt', open('./todo.md', 'rb'), )),
        # ('vul_files_list', ('vul_veinmind.json', open('./todo.md', 'rb'), )),
    ]
    detector = VoyagerDetect(
        token=args.voyager_token if args.voyager_token else VOYAGER_TOKEN,
        url=args.voyager_server if args.voyager_server else VOYAGER_SERVER,
        username=args.voyager_username if args.voyager_username else VOYAGER_USERNAME,
        password=args.voyager_password if args.voyager_password else VOYAGER_PASSWORD,
        api=args.voyager_api if args.voyager_api else ''
    )
    if args.time_wait_for_report:
        detector.set_time_wait_for_report(args.time_wait_for_report)
    if args.error_try_count:
        detector.set_error_try_count(args.error_try_count)
    if args.cmd_time_out:
        detector.set_cmd_time_out(int(args.cmd_time_out))

    # 项目安全检测
    if args.type in ['project_sca']:
        # 逐步进行检测
        for tool in args.tool.split(','):
            detector.sca_analysis(args.source, tool)
        # 打包上传
        zip_folder(DATA_DIR)
        target_file = DATA_DIR.rstrip('/')+'.zip'
        try:
            attach_files.append(
                ('files', (target_file.split('/')[-1], open(target_file, 'rb'),)),
            )
        except Exception as e:
            logger.error(e)
        scan_status = detector.upload(data, attach_files)
        print(scan_status)
    # 服务端执行镜像SCA分析、执行镜像漏洞检测，并获得检测结果
    # image_vul_detect会进行结果的规则检测，返回是否命中，image_sca_detect会对结果文件进行规则检测，返回是否命中
    elif args.type in ['image_vul', 'image_sca', 'image_vul_detect', 'image_sca_detect']:
        image_info = {
            'image_name': args.image_name,
            'image_full_name': args.image_full_name,
            'image_tag': None,
            'image_id': args.image_id,
            'unique_hash': None
        }
        try:
            image_tag = args.image_full_name.split(':')[-1]
            image_tag = args.image_full_name.split(':')[-1]
            image_unique_hash = md5((args.trigger_source + args.image_full_name).encode()).hexdigest()
            image_info['image_tag'] = image_tag
            image_info['unique_hash'] = image_unique_hash
        except Exception as e:
            logger.error(e)
        if args.scan_in_server:
            logger.info('在服务端执行相应的分析操作')
            # image_file/image_dockerfile/image_analysis_file
            if args.image_data_type in ['image_file', 'image_dockerfile', 'image_analysis_file']:
                file_info = None
                if args.image_data_type == 'image_file':
                    file_info = args.image_file
                elif args.image_data_type == 'image_dockerfile':
                    file_info = args.dockerfile_path
                elif args.image_data_type == 'image_analysis_file':
                    file_info = args.image_analysis_file
                if file_info:
                    try:
                        attach_files.append(
                            ('files',
                             (file_info.split('/')[-1].split('.')[0], open(file_info, 'rb'),)),
                        )
                    except Exception as e:
                        logger.error(e)
            elif args.image_data_type == 'image_name':
                pass
            else:
                pass
        else:
            logger.info('在本地执行相应的分析操作')
            if args.image_data_type in ['image_file', 'image_dockerfile', 'image_analysis_file']:
                image_file = args.image_file
                image_dockerfile = args.dockerfile_path
                image_analysis_file = args.image_analysis_file
                # sca部分
                logger.info('开始在本地执行镜像SCA分析')
                for tool in ['syft', 'trivy']:
                    pass
                    detector.image_analysis(src_dir=args.source, tool=tool, analysis_type='sca',
                                            image_info=image_info,
                                            image_file=image_file, sca_result_file=None,
                                            dockerfile_path=None)
                # 可执行文件分析
                detector.image_analysis(src_dir=args.source, tool='custom', analysis_type='exec',
                                        image_info=image_info,
                                        image_file=image_file, sca_result_file=None,
                                        dockerfile_path=None)
                # 获取dockerfile文件
                detector.image_analysis(src_dir=args.source, tool='custom', analysis_type='dockerfile',
                                        image_info=image_info,
                                        image_file=image_file, sca_result_file=None,
                                        dockerfile_path=image_dockerfile)
                if 'image_vul' in args.type:
                    logger.info('开始在本地执行镜像漏洞检测部分（以sca结果进行漏洞检测）')
                    for tool in ['grype', 'trivy']:
                        detector.image_analysis(src_dir=args.source, tool=tool, analysis_type='vul_by_sbom',
                                                image_info=image_info,
                                                image_file=image_file, sca_result_file=None,
                                                dockerfile_path=None)

                zip_folder(DATA_DIR)
                target_file = DATA_DIR.rstrip('/') + '.zip'
                logger.info('将结果文件打包生成的路径为-{}'.format(target_file))
                try:
                    attach_files.append(
                        ('files', (target_file.split('/')[-1], open(target_file, 'rb'),)),
                    )
                except Exception as e:
                    logger.error(e)
            elif args.image_data_type == 'image_name':
                logger.info('本地执行以镜像名进行的镜像检测待实现')

        # 发起相应的检测请求
        logger.info('在服务端执行相应的分析操作，并执行相应的规则检测匹配')
        if attach_files:
            upload_status = detector.upload_files(attach_files)
            file_pk_list = [str(i.get('pk')) for i in upload_status.get('data', [])]
            data['attach_files'] = ','.join(file_pk_list)
        scan_status = detector.detect(data)
        print(scan_status)
    # 单独的扫描结果检测（sbom_syft/sbom_trivy/trivy_vul）
    elif args.type in ['image_scan_result_detect']:
        image_analysis_file = args.image_analysis_file
        if image_analysis_file:
            try:
                attach_files.append(
                    ('files', (image_analysis_file.split('/')[-1], open(image_analysis_file, 'rb'),)),
                )
            except Exception as e:
                logger.error(e)
        if attach_files:
            upload_status = detector.upload_files(attach_files)
            file_pk_list = [str(i.get('pk')) for i in upload_status.get('data', [])]
            data['attach_files'] = ','.join(file_pk_list)
        scan_status = detector.detect(data)
        print(scan_status)
        with open('detect_result.json', 'w') as f:
            try:
                f.write(json.dumps(scan_status))
            except Exception as e:
                logger.error(e)
    else:
        pass
if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
import subprocess
import logging
import os
import json
import sys
import zipfile
import shutil
import requests
import hashlib
import time
from datetime import datetime, timedelta
import plutonium.config as config
from uuid import uuid4
logger = logging.getLogger(__name__)
formatter = logging.Formatter(config.LOG_FORMAT)
file_handler = logging.FileHandler(config.LOG_FILENAME)
file_handler.setFormatter(formatter)
console_handler = logging.StreamHandler()
console_handler.setFormatter(formatter)
logger.addHandler(file_handler)
logger.addHandler(console_handler)
logger.setLevel(config.LOG_LEVEL)
# 获取路径
def resource_path(relative_path):
    try:
        base_path = sys._MEIPASS
    except Exception:
        base_path = os.path.dirname(__file__)
    return os.path.join(base_path, relative_path)
# 执行命令
def exec_tool(args, log_file, timeout=config.CMD_TIME_OUT, cwd=None, ):
    result = {
        'status': False,
        'message': '',
        'data': ''
    }
    try:
        logger.debug('Executing "{}"'.format(" ".join(args)))

        if os.environ.get("FETCH_LICENSE"):
            logger.debug(
                "License information would be fetched from the registry. This would take several minutes ..."
            )
        with open(log_file, 'a') as f:
            cp = subprocess.run(
                args,
                stdout=f,
                stderr=subprocess.STDOUT,
                cwd=cwd,
                env=os.environ.copy(),
                check=False,
                shell=False,
                encoding="utf-8",
                timeout=timeout
            )
            logger.debug(cp.stdout)
            result['status'] = True
    except subprocess.TimeoutExpired as timeout_error:
        logger.error(timeout)
        result['message'] = '执行超时-{}'.format(timeout_error)
    except Exception as e:
        logger.exception(e)
        result['message'] = str(e)
        return str(e)
    print(result)
    return result

def zip_folder(file_dir):
    target_file = file_dir+'/../'+file_dir.strip('/').split('/')[-1]+'.zip'
    print(target_file)
    with zipfile.ZipFile(target_file, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for root, dirs, files in os.walk(file_dir):
            for file in files:
                file_path = os.path.join(root, file)
                zipf.write(file_path, os.path.relpath(file_path, file_dir))
    

# 通过cdxgen来生成sbom
def sca_by_cdxgen(bom_file, src_dir=".", timeout=config.CMD_TIME_OUT):
    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    cdxgen_cmd = os.environ.get("CDXGEN_CMD", "cdxgen")
    if not shutil.which(cdxgen_cmd):
        local_bin = resource_path(
            os.path.join(
                "local_bin", "cdxgen.exe" if sys.platform == "win32" else "cdxgen"
            )
        )
        if not os.path.exists(local_bin):
            result['message'] = 'command not found'
            return result
        try:
            cdxgen_cmd = local_bin
            # Set the plugins directory as an environment variable
            os.environ["CDXGEN_PLUGINS_DIR"] = resource_path("local_bin")
        except Exception as e:
            result['message'] = e
            return result
    sca_args = [cdxgen_cmd, "-o", bom_file]
    sca_args.append(src_dir)
    logger.info(sca_args)
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout )
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result

def sca_by_opensca(bom_file, src_dir=".", log_file="",  timeout=config.CMD_TIME_OUT):
    # 获取配置文件，配置maven等信息

    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    sca_args = ["opensca-cli", "-path", src_dir, "-config", config.TEMP_DIR+'/opensca_config.json', "-out", "{}".format(bom_file)]
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result
def sca_by_jd_sbom_tool(bom_file, src_dir=".", log_file="",  timeout=config.CMD_TIME_OUT):
    # 获取配置文件，配置maven等信息
    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    sca_args = ["sbom-tool", "-out ", bom_file, "-path", src_dir, "-log", log_file]
    logger.info(sca_args)
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result

def sca_by_fosseye(bom_file, src_dir=".", log_file="",  timeout=config.CMD_TIME_OUT):
    # 获取配置文件，配置maven等信息

    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    sca_args = ["opensca-cli", "-out ", bom_file, "-path", src_dir, "-log", log_file]
    logger.info(sca_args)
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result

def sca_by_syft(bom_file, src_dir=".", log_file="",  timeout=config.CMD_TIME_OUT):
    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    sca_args = ["syft", src_dir, "-o", "syft-json={}".format(bom_file)]
    exec_status = exec_tool(sca_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(sca_args)
    return result

def vul_by_grype(result_file, src_dir=".", log_file="", sbom_file="", timeout=config.CMD_TIME_OUT):
    result = {
        'status': False,
        'data': {
            'cmd': '',
            'result_file': bom_file
        },
        'message': '',
    }
    if sbom_file:
        vul_args = ["grype", "sbom:{}".format(sbom_file), "-o", "json", "--file", "{}".format(result_file)]
    else:
        vul_args = ["grype", "dir:{}".format(src_dir), "-o", "json", "--file", "{}".format(result_file)]
    exec_status = exec_tool(vul_args, config.LOG_FILENAME, timeout)
    result['status'] = exec_status['status']
    result['message'] = exec_status['message']
    result['data']['cmd'] = ' '.join(vul_args)
    return result

def vul_by_trivy():
    pass

def vul_by_depscan():
    pass

class ImageFileExtractor(object):
    def __init__(self, tmp_dir='./tmp/', result_dir='./'):
        self.tmp_dir = tmp_dir
        self.result_dir = result_dir

    def calculate_md5(self, file_path):
        md5_hash = hashlib.md5()
        with open(file_path, "rb") as f:
            # Read and update hash string value in blocks of 4K
            for byte_block in iter(lambda: f.read(4096), b""):
                md5_hash.update(byte_block)
        return md5_hash.hexdigest()

    def calculate_sha256(self, file_path):
        sha256_hash = hashlib.sha256()
        with open(file_path, "rb") as f:
            # Read and update hash string value in blocks of 4K
            for byte_block in iter(lambda: f.read(4096), b""):
                sha256_hash.update(byte_block)
        return sha256_hash.hexdigest()

    # 从分层解压目录中获取可执行文件
    def get_executable_files(self, base_folder, layer_id, layer_diff_id):
        logger.info('开始获取可执行文件')
        result_list = []
        for root, dirs, files in os.walk(base_folder):
            for filename in files:
                file_path = os.path.join(root, filename)
                # Check if the file is executable
                if os.access(file_path, os.X_OK):
                    sha256 = self.calculate_sha256(file_path)
                    file_md5 = self.calculate_md5(file_path)
                    file_path = file_path.replace(base_folder, '')
                    try:
                        result_list.append({
                            "filename": filename,
                            "file_path": file_path,
                            "md5": file_md5,
                            "sha256": sha256,
                            "layer_id": layer_id,
                            "layer_diff_id": layer_diff_id,
                            # "unique_hash": md5((filename + file_path + file_md5).encode()).hexdigest()
                        })
                        # unique hash
                    except Exception as e:
                        logger.error(e)
        return result_list

    def run_command(self, command):
        return os.popen(command).read()

    # 解压镜像，并获取可执行文件数据
    def get_execute_file_data_from_image(self, image_info, image_file, result_file):
        result = {
            'status': False,
            'message': '',
            'data': {
                'data': None,
                'file_url': '',
                'file_url_oss': '',
            }
        }
        all_data = {
            'image_full_name': image_info.get('image_full_name'),
            'image_name': image_info.get('image_name'),
            'image_tag': image_info.get('image_tag'),
            'image_id': image_info.get('image_id'),
            'image_unique_hash': image_info.get('unique_hash'),
            'layer_data': {},
            'execute_files': []
        }
        
        image_extract_dir = self.tmp_dir + uuid4().hex
        logger.info('镜像解压存储目录为-{}'.format(image_extract_dir))
        if not os.path.exists(image_extract_dir):
            os.makedirs(image_extract_dir)
        layer_extract_dir = self.tmp_dir + uuid4().hex
        # layer_extract_dir = './tmp/26f0a52d1d8d4fa19bad3f4865f94b88'
        logger.info('镜像分层解压存储目录为-{}'.format(layer_extract_dir))
        if not os.path.exists(layer_extract_dir):
            os.makedirs(layer_extract_dir)
        logger.info('开始解压镜像文件')
        extract_image_command = 'tar -xf {} --directory {}'.format(image_file, image_extract_dir)
        os.system(extract_image_command)
        logger.info('完成解压镜像文件')
        # 删除镜像文件
        logger.info('开始删除镜像tar文件')
        try:
            image_tar_dir = '/'.join(image_file.split('/')[:-1])
            if image_tar_dir.startswith('/data/voyager_sbom/data/'):
                shutil.rmtree(image_tar_dir)
        except Exception as e:
            logger.error(e)

        # docker inspect数据为diff id
        # 镜像解压后的目录名为layer id，里面的layer.tar计算sha256即为diff id
        image_data = None
        try:
            with open(image_extract_dir + '/manifest.json', 'r') as f:
                image_data = json.loads(f.read())
        except Exception as e:
            logger.error(e)
        layer_data = {}
        layer_file_map = {}
        # key为diff id(layer id sha256)
        # 值为layer id
        if image_data and len(image_data) > 1:
            logger.info("镜像manifest数据大于一个，需排查")
        try:
            for layer in image_data[0].get('Layers'):
                # 有的Layers的结尾是layer_id+'.tar'
                # 数据中有如下几种格式
                # d0241a0d0759b0d4adbfc4d9278479e28d4762abaaa21ba4135fc58710dbd88d/layer.tar格式
                # 0ad3ddf4a4ce0fc9d65021897541c99e5497a11ced2419016147267df61732d3.tar
                # "blobs/sha256/8a99c598a0e4dd63ed2ab07cb5ba08b51988ba78dd022c4c28a24ed40f91de5a
                if sys.platform == 'darwin':
                    output = self.run_command("ls {}/{} | xargs shasum -a 256".format(image_extract_dir, layer))
                else:
                    output = self.run_command("ls {}/{}| xargs sha256sum".format(image_extract_dir, layer))
                diff_id = output.split()[0].strip()

                # oci镜像
                if layer.startswith('blobs/sha256'):
                    layer_id = layer.split('/')[-1]
                else:
                    if layer.endswith('/layer.tar'):
                        layer_id = layer.split('/')[0]
                    else:
                        layer_id = layer.split('.')[0]
                layer_file_map[diff_id] = layer
                layer_data[diff_id] = layer_id
        except Exception as e:
            logger.error(e)
        # 分层数据
        all_data['layer_data'] = layer_data
        logger.info('分层数据为[diff id] -> [layer id]')
        logger.info('开始解压分层文件')
        all_file_data = {}
        for diff_id in layer_data:
            # 创建新的目录，目录为diff id，里面存储解压文件内容
            layer_item_dir = layer_extract_dir + '/' + layer_data[diff_id]
            if not os.path.exists(layer_item_dir):
                os.makedirs(layer_item_dir)
            # os.system(f"mkdir -p {dest}")
            logger.info('开始解压分层-diff id is {}'.format(diff_id))
            logger.info('开始解压分层layer id为-{}，diff id为-{}'.format(layer_data[diff_id], diff_id))
            os.system("tar -xf {}/{} --directory {}".format(image_extract_dir, layer_file_map[diff_id], layer_item_dir))
            logger.info("tar -xf {}/{} --directory {}".format(image_extract_dir, layer_file_map[diff_id], layer_item_dir))
            # 遍历全部文件内容，获取可执行文件、路径、sha256
            execute_files = self.get_executable_files(layer_item_dir, layer_data[diff_id], diff_id)
            all_file_data[diff_id] = execute_files
        logger.info("完成分析")
        all_data['execute_files'] = all_file_data
        # 写入数据到文件
        # 清理解压
        try:
            shutil.rmtree(image_extract_dir)
            shutil.rmtree(layer_extract_dir)
        except Exception as e:
            logger.error(e)
        # 写入数据到文件
        try:
            with open(result_file, 'w') as f:
                f.write(json.dumps(all_data))
                result['status'] = True
                result['data']['data'] = all_data
                result['data']['file_url'] = result_file
        except Exception as e:
            logger.error(e)
            result['message'] = str(e)

        return result


# 执行对镜像的分析，并输出结果文件


# 服务端检测
class VoyagerDetect():
    def __init__(self, token=None, url=None, username=None, password=None, api=None):
        self.api_url = url
        self.api_token = token
        self.api_username = username
        self.api_password = password
        self.req = requests.Session()
        self.error_try_count = 5
        self.req_time_out = 60
        self.time_wait_for_report = 10
        self.cmd_time_out = 10
    def set_req_time_out(self, time_seconds):
        self.req_time_out = time_seconds
    def set_error_try_count(self, count):
        self.error_try_count = count

    def set_time_wait_for_report(self, time_seconds):
        self.time_wait_for_report = time_seconds
    def set_cmd_time_out(self,time_seconds):
        self.cmd_time_out = time_seconds
        

    # 1.token生成，token有效期较长，失效后再进行重新生成
    def get_new_token(self):
        try:
            self.req.headers = {}
            res = self.req.post(self.api_url + config.VOYAGER_GET_TOKEN_API,
                                data={'username': self.api_username, 'password': self.api_password},)
            token = res.json()['token']
            headers = {
                # 注意Token后有空格
                'Authorization': 'Token ' + token
            }
            self.req.headers = headers
            return token
        except Exception as e:
            logger.error(e)
            return None

    def check_token_valid(self):
        headers = {
            # 注意Token后有空格
            'Authorization': 'Token ' + self.api_token
        }
        self.req.headers = headers
        try:
            res = self.req.get(self.api_url + config.VOYAGER_CHECK_TOKEN_API)
            if res.status_code == 200 and res.json()['success']:
                logger.info('token有效')
                return True
            else:
                logger.error('token无效')
                logger.info(res.text)
        except Exception as e:
            logger.info('token失效')
            logger.error(e)
        return False
    # 登录认证
    def login(self):
        login_status = {
            'status': False,
            'message': '',
            'data': None
        }
        if self.api_token:
            if self.check_token_valid():
                login_status['status'] = True
                return login_status
            else:
                # token失效，使用账号密码登录
                login_status['message'] = 'token失效，使用账号密码登录'

        if self.api_username and self.api_password:
            # 获取认证token
            token = self.get_new_token()
            # logger.info(token)
            if token:
                self.api_token = token
                headers = {
                    # 注意Token后有空格
                    'Authorization': 'Token ' + self.api_token
                }
                self.req.headers = headers
                login_status['status'] = True
            else:
                login_status['message'] = '无法生成访问token，账号密码可能错误'
        else:
            login_status['message'] = '请提供API账号以及密码信息'
        logger.info(login_status['message'])
        return login_status

    def init_opensca(self):
        login_status = self.login()
        data = {
            'type': 'get_opensca_config',
        }
        if login_status['status']:
            response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data,
                                         files=[('files', ('', None,)), ])
            try:
                print(response.json())
                if response.json().get('success'):
                    config_data = response.json().get('data')
                    print(config.TEMP_DIR+'opensca_config.json')
                    with open(config.TEMP_DIR+'/opensca_config.json', 'w') as f:
                        f.write(json.dumps(config_data))
            except Exception as e:
                logger.error(e)
        else:
            return login_status['message']

    # sca分析
    def sca_analysis(self, src_dir=".", tool='cdxgen'):
        if tool == 'cdxgen':
            result_file = config.DATA_DIR+'/{}_{}.json'.format(tool, uuid4().hex)
            return sca_by_cdxgen(result_file, src_dir)
        elif tool == 'opensca':
            self.init_opensca()
            result_file = config.DATA_DIR+'/{}_{}.json'.format(tool, uuid4().hex)
            log_file = config.DATA_DIR+'/{}_log_{}.log'.format(tool, uuid4().hex)
            return sca_by_opensca(result_file, src_dir, log_file)
        elif tool == 'syft':
            result_file = config.DATA_DIR + '/{}_{}.json'.format(tool, uuid4().hex)
            log_file = config.DATA_DIR + '/{}_log_{}.log'.format(tool, uuid4().hex)
            return sca_by_syft(result_file, src_dir, log_file)
        elif tool == 'grype':
            result_file = config.DATA_DIR + '/{}_{}.json'.format(tool, uuid4().hex)
            log_file = config.DATA_DIR + '/{}_log_{}.log'.format(tool, uuid4().hex)
            sbom_result_file = None
            try:
                for i in os.listdir(config.DATA_DIR):
                    if i.startswith('syft_'):
                        sbom_result_file = config.DATA_DIR + '/' + i
                        break
            except Exception as e:
                logger.error(e)
            return vul_by_grype(result_file, src_dir, log_file, sbom_result_file)
        elif tool == 'jd-sbom-tool':
            pass

        return None

    # 上传文件获得文件id
    def upload_files(self,files):
        data = {
            'type': 'file'
        }
        login_status = self.login()
        if login_status['status']:
            try:
                response = self.req.post(self.api_url + config.VOYAGER_UPLOAD_API, data=data, files=files, timeout=self.req_time_out)
                return response.json()
            except Exception as e:
                logger.error(e)
                return None
        else:
            return login_status['message']
    # upload
    def upload(self, data, files):
        login_status = self.login()
        if login_status['status']:
            try:
                if files:
                    response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data, files=files, timeout=self.req_time_out)
                else:
                    response = self.req.post(self.api_url + config.VOYAGER_BASE_API, data=data,
                                             files=[('files', ('', None,)), ], timeout=self.req_time_out)
                return response.json()
            except Exception as e:
                logger.error(e)
        else:
            return login_status['message']

    # 获取扫描结果或报告
    def get_scan_result(self, task_id):
        login_status = self.login()
        if login_status['status']:
            try:
                response = self.req.get(self.api_url + config.VOYAGER_BASE_API+'?task_id={}'.format(task_id),)
                return response.json()
            except Exception as e:
                logger.error(e)
                return None
        else:
            return login_status
    def get_detect_result(self, task_id):
        login_status = self.login()
        if login_status['status']:
            try:
                response = self.req.get(self.api_url + config.VOYAGER_DETECT_API+'?task_id={}'.format(task_id),)
                return response.json()
            except Exception as e:
                logger.error(e)
                return None
        else:
            return login_status
    # 根据镜像名、镜像文件在服务端进行镜像漏洞检测
    def scan_image_in_server(self, data, files):
        login_status = self.login()
        if login_status['status']:
            if files:
                response = self.req.post(self.api_url + config.VOYAGER_DETECT_API, data=data, files=files)
            else:
                response = self.req.post(self.api_url + config.VOYAGER_DETECT_API, data=data, files=[('files', ('', None,)), ])
            return response.json()
        else:
            return login_status['message']

    def image_analysis(self, src_dir='.', tool='custom', analysis_type='sca', image_info={}, image_file=None, sca_result_file=None,
                       dockerfile_path=None):
        result = {
            'status': False,
            'data': {
                'cmd': '',
                'result_file': ''
            },
            'message': '',
        }
        op_msg = ''
        cmd_args = []
        result_file = config.DATA_DIR+'/{}--{}--result.json'.format(analysis_type, tool)
        # 镜像文件
        if not image_file:
            image_file = src_dir + '/' + 'image.tar'

        if analysis_type == 'sca':
            if tool == 'syft':
                op_msg = '基于syft进行镜像的SCA分析'
                cmd_args = ["syft", image_file, "-o", "syft-json={}".format(result_file)]
            elif tool == 'trivy':
                op_msg = '基于trivy进行镜像的SCA分析'
                # trivy image --format cyclonedx --output /tmp/sca_trivy_result.json --input /tmp/hi.tar
                cmd_args = ["trivy", "image", "--format", "cyclonedx", "--output", result_file, "--input", image_file]
            else:
                pass
        elif analysis_type == 'vul_by_sbom':
            if tool == 'grype':
                op_msg = '基于grype进行镜像的漏洞分析'
                sca_file = result_file.format('sca', 'syft')
                cmd_args = ["grype", "sbom:{}".format(sca_file), "-o", "json", "--file", result_file]

            elif tool == 'trivy':
                op_msg = '基于trivy进行镜像的漏洞分析'
                sca_file = result_file.format('sca', 'trivy')
                cmd_args = ["trivy", "--scanners", "vuln,misconfig,secret", "sbom", sca_file, "--format json", "-o",
                            result_file]
            else:
                pass
        elif analysis_type == 'vul':
            if tool == 'grype':
                op_msg = '基于grype进行镜像的漏洞分析'
                cmd_args = ["grype", image_file, "-o", "json", "--file", result_file]

            elif tool == 'trivy':
                op_msg = '基于trivy进行镜像的漏洞分析'
                cmd_args = ["trivy", "image", "--scanners", "vuln,misconfig,secret", "--format json", "-o", result_file,
                            "--input",
                            image_file]
            else:
                pass
        elif analysis_type == 'exec':
            image_extractor = ImageFileExtractor()
            op_msg = '镜像可执行文件分析提取'
            logger.info('[开始]' + op_msg)
            exec_files_result = image_extractor.get_execute_file_data_from_image(image_info, image_file, result_file)
            if exec_files_result['status']:
                result['status'] = True
                logger.info('[完成]' + op_msg)
            else:
                logger.info('[异常]' + op_msg)
                logger.info(exec_files_result['message'])
                result['message'] = exec_files_result['message']
        elif analysis_type == 'backdoor':
            pass
        elif analysis_type == 'dockerfile':
            op_msg = '读取Dockerfile文件内容'
            logger.info('[开始]' + op_msg)
            # 读取该文件，并写入结果文件
            if not dockerfile_path:
                dockerfile_path = src_dir + '/' + 'Dockerfile'
            if not os.path.exists(dockerfile_path):
                result['message'] = '不存在Dockerfile文件'
                logger.info('[异常]' + op_msg)
            try:
                with open(dockerfile_path, 'r') as f:
                    data = f.read()
                    with open(result_file, 'w') as w:
                        w.write(data)
                        logger.info('[完成]' + op_msg)
                        result['status'] = True
            except Exception as e:
                logger.error(e)
                logger.info('[异常]' + op_msg)
                result['message'] = str(e)
        else:
            logger.info('分析类型待扩展')
        logger.info('[开始]' + op_msg)
        logger.info(cmd_args)
        if len(cmd_args) >0:
            exec_status = exec_tool(cmd_args, config.LOG_FILENAME, self.cmd_time_out)
            if exec_status['status']:
                logger.info('[完成]' + op_msg)
            else:
                logger.info('[异常]' + op_msg)
                logger.info(exec_status['message'])
        result['data']['cmd'] = ' '.join(cmd_args)
        result['data']['result_file'] = result_file
        return result

    def make_detect_req(self, data):
        login_status = self.login()
        if login_status['status']:
            try:
                res = self.req.post(self.api_url+config.VOYAGER_DETECT_API, data=data,)
                return res.json()
            except Exception as e:
                logger.error(e)
                return None
        else:
            return login_status['message']

    # 添加超时时间，命令执行超时时间、检测耗时超时时间
    def detect(self, data,):
        # 默认安全检测为通过，即安全服务异常情况下其允许通过
        result = {
            'passed': True,
            'message': '',
            'data': None
        }
        scan_status = self.make_detect_req(data)
        try_count = 1
        while try_count <= self.error_try_count:
            if scan_status:
                if scan_status['success']:
                    task_id = scan_status['data']
                    scan_run_time = datetime.now()
                    # 轮询获取扫描结果，直到任务完成或超时时间超过5分钟
                    while True:
                        scan_result = self.get_detect_result(task_id)
                        logger.info(scan_result)
                        if scan_result:
                            try:
                                # 代表任务执行完成
                                if scan_result['data']['status'] in ['success', 'error']:
                                    result['passed'] = scan_result['data']['passed']
                                    result['message'] = scan_result['data']['message']
                                    result['data'] = scan_result['data']['data']
                                    break
                                else:
                                    logger.info('任务正在执行，等待尝试继续查询')
                                    time.sleep(10)
                            except Exception as e:
                                logger.error(e)
                                time.sleep(10)
                        else:
                            logger.info('调用查看检测结果服务异常，等待尝试基线查询')
                            time.sleep(10)

                        if datetime.now() > scan_run_time + timedelta(seconds=int(self.time_wait_for_report)):
                            logger.info('等待超时，结束服务调用')
                            break
                    break
                else:
                    logger.info('调用扫描服务失败-{}'.format(scan_status['message']))
                    try_count += 1
                    time.sleep(10)
            else:
                try_count += 1
                logger.info('调用扫描服务请求异常，正在重新尝试第{}次'.format(try_count))
                time.sleep(10)
                scan_status = self.make_detect_req(data)
        if result['passed']:
            logger.info('当前镜像检测通过，可以继续执行流水线')
            logger.info(result['message'])
        else:
            logger.info('当前镜像检测未通过，流水线终止执行，可查看相关加白申请')
            logger.info(result['message'])
            logger.info(result['data'])
        return result

    # 镜像可执行文件提取
    # 镜像sca分析，使用syft/trivy并将结果上传
    # 镜像sca分析以及漏洞检测，并将结果上传
    